// Arduino WebPage LED strip control server
// Author: Robotics & Energy
// April,  2023

#include <SPI.h>
#include <Ethernet.h>
#include <SD.h>

#define REQ_BUF_SZ 50 // size of buffer used to capture HTTP requests

//RGB LED Pins
#define RED 3
#define GREEN 5
#define BLUE 6
#define PWR 2
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
#define IP IPAddress(192,168,0,12) //Gateway: 192.168.0.1

EthernetServer server(80);       // create a server at port 80
File webFile;                    // handle to files on SD card
char HTTP_req[REQ_BUF_SZ] = {0}; // buffered HTTP request stored as null terminated string
byte req_index = 0;              // index into HTTP_req buffer
bool root = false;               // login indicator

//---------------------------------------------<functions>-------------------------------------------------------------------

// sets every element of str to 0 (clears array)
void StrClear(char *str, char len)
{
    for (int i = 0; i < len; i++) 
        str[i] = 0;
}

// searches for the string sfind in the string str
// returns position if string found
// returns 0 if string not found
byte StrContains(char *str, char *sfind)
{
    //byte len = strlen(str); 
    #define LEN strlen(str) 
    byte found = 0;
    byte index = 0;
      
    if (strlen(sfind) > LEN) return 0;
    while (index < LEN) 
    {
        if (str[index] == sfind[found]) 
        {
            found++;
            if (strlen(sfind) == found) return index;    
        }
        else found = 0;
        index++;
    }
    return 0;
}

void controlHandler(unsigned long int dataRGB) // dataRGB = 1255255255 (VRRRGGGBBB)
{   
    byte P =(byte)((dataRGB / 1000000000)      );
    byte R =(byte)((dataRGB / 1000000)   % 1000);
    byte G =(byte)((dataRGB / 1000)      % 1000); 
    byte B =(byte)((dataRGB)             % 1000);  
    
    digitalWrite(PWR,   P);
    analogWrite(RED,    R);
    analogWrite(GREEN,  G); 
    analogWrite(BLUE,   B);

}
//-----------------------------------------</functions>-------------------------------------------------------------------

void setup()
{
    //Outputs
    pinMode(13, OUTPUT);
    pinMode(RED, OUTPUT);
    pinMode(GREEN, OUTPUT);
    pinMode(BLUE, OUTPUT);
    pinMode(PWR, OUTPUT);

    digitalWrite(13, LOW);
    digitalWrite(PWR, LOW);
    analogWrite(RED, 0);
    analogWrite(GREEN, 0);
    analogWrite(BLUE, 0);
       
    Serial.begin(230400);   // for debugging, also defins the connection speed so make it big!   
    // initialize SD card
    Serial.println("Initializing SD card...");
    if (!SD.begin(4)) 
    {
        Serial.println("ERROR - SD card initialization failed!");
        return;    // init failed
    }
    Serial.println("SUCCESS - SD card initialized.");
    Ethernet.begin(mac, IP);  // initialize Ethernet device => Ethernet.begin(mac, ip);
    server.begin();           // start to listen for clients
}

void loop()
{ 
    EthernetClient client = server.available();  // try to get client

    if (client) // got client?
    {  
        bool currentLineIsBlank = true;
        while (client.connected()) 
        {
            if (client.available()) // client data available to read
            {   
                char c = client.read(); // read 1 byte (character) from client
                // buffer first part of HTTP request in HTTP_req array (string)
                // leave last element in array as 0 to null terminate string (REQ_BUF_SZ - 1)
                if (req_index < (REQ_BUF_SZ - 1)) 
                {
                    HTTP_req[req_index] = c;          // save HTTP request character
                    req_index++;
                }
                Serial.print(c);    // print HTTP request character to serial monitor
                // last line of client request is blank and ends with \n
                // respond to client only after last line received
                if (c == '\n' && currentLineIsBlank) 
                { 
                    if(root)
                    {
                      if(StrContains(HTTP_req, "GET / ") || StrContains(HTTP_req, "GET /login.htm"))                            
                      {
                          root = false;
                          client.println("HTTP/1.1 200 OK");
                          client.println("Content-Type: text/html");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("login.htm");      
                      }
                      else if(StrContains(HTTP_req, "control.htm"))
                      {
                          root = true;
                          client.println("HTTP/1.1 200 OK");
                          client.println("Content-Type: text/html");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("control.htm");        
                      }
                      else if(StrContains(HTTP_req, "main.htm"))
                      {
                          root = true;
                          client.println("HTTP/1.1 200 OK");
                          client.println("Content-Type: text/html");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("main.htm");        
                      }
                      else if (StrContains(HTTP_req, "elec.jpg")) 
                      {
                          client.println("HTTP/1.1 200 OK");
                          client.println();
                          webFile = SD.open("elec.jpg");
                      }
                      else if (StrContains(HTTP_req, "style.css")) 
                      {
                          client.println("HTTP/1.1 200 OK");
                          client.println("Content-Type: text/css");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("style.css");
                      }
                      else if (StrContains(HTTP_req, "click?")) 
                      {
                          client.println("HTTP/1.1 200 OK");
                          client.println();
                          // Get RGB values from the HTTP packet
                          unsigned long int result = 0; 
                          sscanf(HTTP_req, "GET /click?=%lux HTTP/1.1", &result);
                          controlHandler(result);
                      }
                      else
                      {
                          client.println("HTTP/4.5 404 Not Found");
                          client.println("Content-Type: text/html");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("404.htm");
                      }
                    }
                    else
                    {
                      if(StrContains(HTTP_req, "GET / ") || StrContains(HTTP_req, "GET /login.htm"))      
                      {
                          client.println("HTTP/1.1 200 OK");
                          client.println("Content-Type: text/html");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("login.htm");      
                      }
                      else if(StrContains(HTTP_req, "user=root&pass=1234"))
                      {
                          root = true;
                          client.println("HTTP/1.1 200 OK");
                          client.println("Content-Type: text/html");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("main.htm");        
                      }
                      else if (StrContains(HTTP_req, "style.css")) 
                      {
                          client.println("HTTP/1.1 200 OK");
                          client.println("Content-Type: text/css");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("style.css");
                      } 
                      else
                      {
                          client.println("HTTP/4.2 401 Unauthorized");
                          client.println("Content-Type: text/html");
                          client.println("Connnection: close");
                          client.println();
                          webFile = SD.open("401.htm");
                      }
                    }
                   if (webFile) 
                   {
                        while(webFile.available()) 
                        {
                            client.write(webFile.read()); // send web page to client
                        }
                        webFile.close();
                    }                 
                    // reset buffer index and all buffer elements to 0
                    req_index = 0;
                    StrClear(HTTP_req, REQ_BUF_SZ);
                    break;
                }
                // every line of text received from the client ends with \r\n
                if (c == '\n') 
                {
                    // last character on line of received text
                    // starting new line with next character read
                    currentLineIsBlank = true;
                } 
                else if (c != '\r') 
                {
                    // a text character was received from client
                    currentLineIsBlank = false;
                }
            } // end if (client.available())
        } // end while (client.connected())
        delay(1);      // give the web browser time to receive the data
        client.stop(); // close the connection
    } // end if (client)
}
